@@ -1,11 +1,11 @@
-#ifndef __DIRECTORYLISTING_H__
-#define __DIRECTORYLISTING_H__
+#ifndef FILEZILLA_ENGINE_DIRECTORYLISTING_HEADER
+#define FILEZILLA_ENGINE_DIRECTORYLISTING_HEADER
 
 #include <libfilezilla/optional.hpp>
 #include <libfilezilla/shared.hpp>
 #include <libfilezilla/time.hpp>
 
-#include <map>
+#include <unordered_map>
 
 class CDirentry
 {
@@ -73,18 +73,18 @@ public:
 	CDirectoryListing& operator=(CDirectoryListing const&) = default;
 	CDirectoryListing& operator=(CDirectoryListing &&) noexcept = default;
 
-	const CDirentry& operator[](unsigned int index) const;
+	CDirentry const& operator[](unsigned int index) const;
 
 	// Word of caution: You MUST NOT change the name of the returned
 	// entry if you do not call ClearFindMap afterwards
-	CDirentry& operator[](unsigned int index);
+	CDirentry& get(unsigned int index);
 
 	unsigned int GetCount() const { return m_entries ? m_entries->size() : 0; }
 
 	void Append(CDirentry&& entry);
 
 	int FindFile_CmpCase(std::wstring const& name) const;
-	int FindFile_CmpNoCase(wxString name) const;
+	int FindFile_CmpNoCase(std::wstring const& name) const;
 
 	void ClearFindMap();
 
@@ -126,7 +126,7 @@ public:
 	bool has_perms() const { return (m_flags & listing_has_perms) != 0; }
 	bool has_usergroup() const { return (m_flags & listing_has_usergroup) != 0; }
 
-	void Assign(std::deque<fz::shared_value<CDirentry>> & entries);
+	void Assign(std::vector<fz::shared_value<CDirentry>> && entries);
 
 	bool RemoveEntry(unsigned int index);
 
@@ -136,8 +136,11 @@ protected:
 
 	fz::shared_optional<std::vector<fz::shared_value<CDirentry> > > m_entries;
 
-	mutable fz::shared_optional<std::multimap<std::wstring, unsigned int> > m_searchmap_case;
-	mutable fz::shared_optional<std::multimap<std::wstring, unsigned int> > m_searchmap_nocase;
+	mutable fz::shared_optional<std::unordered_multimap<std::wstring, unsigned int> > m_searchmap_case;
+	mutable fz::shared_optional<std::unordered_multimap<std::wstring, unsigned int> > m_searchmap_nocase;
 };
 
+// Checks if listing2 is a subset of listing1. Compares only filenames.
+bool CheckInclusion(CDirectoryListing const& listing1, CDirectoryListing const& listing2);
+
 #endif
