@@ -1,5 +1,5 @@
-#ifndef __LOCALTREEVIEW_H__
-#define __LOCALTREEVIEW_H__
+#ifndef FILEZILLA_INTERFACE_LOCALTREEVIEW_HEADER
+#define FILEZILLA_INTERFACE_LOCALTREEVIEW_HEADER
 
 #include <option_change_event_handler.h>
 #include "systemimagelist.h"
@@ -13,10 +13,8 @@ class CWindowTinter;
 class CVolumeDescriptionEnumeratorThread;
 #endif
 
-class CLocalTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
+class CLocalTreeView final : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
 {
-	DECLARE_CLASS(CLocalTreeView)
-
 	friend class CLocalTreeViewDropTarget;
 
 public:
@@ -31,7 +29,7 @@ public:
 protected:
 	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 
-	void SetDir(wxString localDir);
+	void SetDir(wxString const& localDir);
 	void RefreshListing();
 
 #ifdef __WXMSW__
