@@ -154,9 +154,8 @@ public:
 			return wxDragNone;
 		}
 
-		const std::list<CRemoteDataObject::t_fileInfo>& files = m_pRemoteDataObject->GetFiles();
-		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
-			const CRemoteDataObject::t_fileInfo& info = *iter;
+		const std::vector<CRemoteDataObject::t_fileInfo>& files = m_pRemoteDataObject->GetFiles();
+		for (auto const& info : files) {
 			if (info.dir) {
 				CServerPath dir = m_pRemoteDataObject->GetServerPath();
 				dir.AddSegment(info.name);
@@ -170,8 +169,7 @@ public:
 			}
 		}
 
-		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
-			const CRemoteDataObject::t_fileInfo& info = *iter;
+		for (auto const& info : files) {
 			m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
 				new CRenameCommand(m_pRemoteDataObject->GetServerPath(), info.name, target, info.name)
 				);
@@ -493,8 +491,9 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		if (entry.is_dir()) {
 			data.icon = m_dirIcon;
 #ifndef __WXMSW__
-			if (entry.is_link())
+			if (entry.is_link()) {
 				data.icon += 3;
+			}
 #endif
 		}
 		m_fileData.push_back(data);
@@ -504,10 +503,12 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		}
 
 		if (m_pFilelistStatusBar) {
-			if (entry.is_dir())
+			if (entry.is_dir()) {
 				m_pFilelistStatusBar->AddDirectory();
-			else
+			}
+			else {
 				m_pFilelistStatusBar->AddFile(entry.size);
+			}
 		}
 
 		// Find correct position in index mapping
@@ -2122,7 +2123,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 	bool idle = m_state.m_pCommandQueue->Idle();
 
 	long item = -1;
-	int count = 0;
+	size_t count = 0;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
@@ -2158,6 +2159,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 	CServerPath const path = m_pDirectoryListing->path;
 
 	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(server, m_pDirectoryListing->path);
+	pRemoteDataObject->Reserve(count);
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
