@@ -1,5 +1,5 @@
-#ifndef __DNDOBJECTS_H__
-#define __DNDOBJECTS_H__
+#ifndef FILEZILLA_INTERFACE_DNDOBJECTS_HEADER
+#define FILEZILLA_INTERFACE_DNDOBJECTS_HEADER
 
 #ifdef __WXMSW__
 #define FZ3_USESHELLEXT 1
@@ -13,7 +13,7 @@
 
 #include <memory>
 
-class wxRemoteDataFormat : public wxDataFormat
+class wxRemoteDataFormat final : public wxDataFormat
 {
 public:
 	wxRemoteDataFormat()
@@ -22,10 +22,10 @@ public:
 	}
 };
 
-class CRemoteDataObject : public wxDataObjectSimple
+class CRemoteDataObject final : public wxDataObjectSimple
 {
 public:
-	CRemoteDataObject(const CServer& server, const CServerPath& path);
+	CRemoteDataObject(ServerWithCredentials const& server, const CServerPath& path);
 	CRemoteDataObject();
 
 	virtual size_t GetDataSize() const;
@@ -38,33 +38,34 @@ public:
 
 	bool DidSendData() const { return m_didSendData; }
 
-	const CServer& GetServer() const { return m_server; }
+	ServerWithCredentials const& GetServer() const { return server_; }
 	const CServerPath& GetServerPath() const { return m_path; }
 	int GetProcessId() const { return m_processId; }
 
 	struct t_fileInfo
 	{
-		wxString name;
-		bool dir;
+		std::wstring name;
 		int64_t size;
+		bool dir;
 		bool link;
 	};
 
-	const std::list<t_fileInfo>& GetFiles() const { return m_fileList; }
+	const std::vector<t_fileInfo>& GetFiles() const { return m_fileList; }
 
+	void Reserve(size_t count);
 	void AddFile(const wxString& name, bool dir, int64_t size, bool link);
 
 protected:
-	CServer m_server;
+	ServerWithCredentials server_;
 	CServerPath m_path;
 
 	mutable CXmlFile m_xmlFile;
 
-	bool m_didSendData;
+	bool m_didSendData{};
 
 	int m_processId;
 
-	std::list<t_fileInfo> m_fileList;
+	std::vector<t_fileInfo> m_fileList;
 
 	mutable size_t m_expectedSize{};
 };
@@ -99,6 +100,6 @@ protected:
 	std::wstring m_dragDirectory;
 };
 
-#endif //__WXMSW__
+#endif
 
-#endif //__DNDOBJECTS_H__
+#endif
