@@ -1,5 +1,5 @@
-#ifndef __FILTER_H__
-#define __FILTER_H__
+#ifndef FILEZILLA_INTERFACE_FILTER_HEADER
+#define FILEZILLA_INTERFACE_FILTER_HEADER
 
 #include "dialogex.h"
 
@@ -29,6 +29,8 @@ class CFilterCondition
 public:
 	CFilterCondition();
 
+	bool CompileRegex();
+
 	t_filterType type;
 	int condition;
 
@@ -50,6 +52,10 @@ public:
 		not_all
 	};
 
+	bool empty() const { return filters.empty(); }
+
+	explicit operator bool() const { return !filters.empty(); }
+
 	wxString name;
 
 	bool filterFiles{true};
@@ -84,10 +90,10 @@ class CFilterManager
 {
 public:
 	CFilterManager();
-	virtual ~CFilterManager() {}
+	virtual ~CFilterManager() = default;
 
 	// Note: Under non-windows, attributes are permissions
-	bool FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const;
+	virtual bool FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const;
 	bool FilenameFiltered(std::vector<CFilter> const& filters, std::wstring const& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date) const;
 	static bool FilenameFilteredByFilter(CFilter const& filter, std::wstring const& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date);
 	static bool HasActiveFilters(bool ignore_disabled = false);
@@ -123,11 +129,10 @@ protected:
 };
 
 class CMainFrame;
-class CFilterDialog : public wxDialogEx, public CFilterManager
+class CFilterDialog final : public wxDialogEx, public CFilterManager
 {
 public:
 	CFilterDialog();
-	virtual ~CFilterDialog() {}
 
 	bool Create(CMainFrame* parent);
 
@@ -150,13 +155,13 @@ protected:
 
 	void OnChangeAll(wxCommandEvent& event);
 
-	bool m_shiftClick;
+	bool m_shiftClick{};
 
-	CMainFrame* m_pMainFrame;
+	CMainFrame* m_pMainFrame{};
 
 	std::vector<CFilter> m_filters;
 	std::vector<CFilterSet> m_filterSets;
 	unsigned int m_currentFilterSet;
 };
 
-#endif //__FILTER_H__
+#endif
