@@ -30,7 +30,7 @@ CFilterEditDialog::~CFilterEditDialog()
 	}
 }
 
-void CFilterEditDialog::OnOK(wxCommandEvent& event)
+void CFilterEditDialog::OnOK(wxCommandEvent&)
 {
 	if (!Validate())
 		return;
@@ -52,7 +52,7 @@ void CFilterEditDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CFilterEditDialog::OnCancel(wxCommandEvent& event)
+void CFilterEditDialog::OnCancel(wxCommandEvent&)
 {
 	EndModal(wxID_CANCEL);
 }
@@ -168,29 +168,28 @@ void CFilterEditDialog::OnNew(wxCommandEvent&)
 	OnFilterSelect(evt);
 }
 
-void CFilterEditDialog::OnDelete(wxCommandEvent& event)
+void CFilterEditDialog::OnDelete(wxCommandEvent&)
 {
 	int item = m_pFilterListCtrl->GetSelection();
-	if (item == -1)
+	if (item == -1) {
 		return;
+	}
 
 	m_currentSelection = -1;
 	m_pFilterListCtrl->Delete(item);
 	m_filters.erase(m_filters.begin() + item);
 
 	// Remote filter from all filter sets
-	for (auto iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
-	{
-		CFilterSet& set = *iter;
+	for (auto & set : m_filterSets) {
 		set.local.erase(set.local.begin() + item);
 		set.remote.erase(set.remote.begin() + item);
 	}
 
-	ClearFilter(true);
+	ClearFilter();
 	SetCtrlState(false);
 }
 
-void CFilterEditDialog::OnRename(wxCommandEvent& event)
+void CFilterEditDialog::OnRename(wxCommandEvent&)
 {
 	const wxString& oldName = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
 	wxTextEntryDialog *pDlg = new wxTextEntryDialog(this, _("Please enter a new name for the filter."), _("Enter filter name"), oldName);
@@ -223,7 +222,7 @@ void CFilterEditDialog::OnRename(wxCommandEvent& event)
 	m_pFilterListCtrl->Select(m_currentSelection);
 }
 
-void CFilterEditDialog::OnCopy(wxCommandEvent& event)
+void CFilterEditDialog::OnCopy(wxCommandEvent&)
 {
 	if (m_currentSelection == -1)
 		return;
@@ -274,11 +273,10 @@ void CFilterEditDialog::OnCopy(wxCommandEvent& event)
 	OnFilterSelect(evt);
 }
 
-void CFilterEditDialog::OnFilterSelect(wxCommandEvent& event)
+void CFilterEditDialog::OnFilterSelect(wxCommandEvent&)
 {
 	int item = m_pFilterListCtrl->GetSelection();
-	if (item == -1)
-	{
+	if (item == -1) {
 		m_currentSelection = -1;
 		SetCtrlState(false);
 		return;
@@ -289,8 +287,7 @@ void CFilterEditDialog::OnFilterSelect(wxCommandEvent& event)
 	if (item == m_currentSelection)
 		return;
 
-	if (m_currentSelection != -1)
-	{
+	if (m_currentSelection != -1) {
 		wxASSERT((unsigned int)m_currentSelection < m_filters.size());
 
 		if (!Validate())
@@ -314,6 +311,7 @@ void CFilterEditDialog::OnFilterSelect(wxCommandEvent& event)
 
 void CFilterEditDialog::SetCtrlState(bool enabled)
 {
+	XRCCTRL(*this, "ID_CASE", wxCheckBox)->Enable(enabled);
 	XRCCTRL(*this, "ID_FILES", wxCheckBox)->Enable(enabled);
 	XRCCTRL(*this, "ID_DIRS", wxCheckBox)->Enable(enabled);
 }
