@@ -1,11 +1,11 @@
-#ifndef __QUEUE_STORAGE_H__
-#define __QUEUE_STORAGE_H__
+#ifndef FILEZILLA_INTERFACE_QUEUE_STORAGE_HEADER
+#define FILEZILLA_INTERFACE_QUEUE_STORAGE_HEADER
 
 #include <vector>
 
 class CFileItem;
 class CServerItem;
-class CServer;
+class ServerWithCredentials;
 
 class CQueueStorage
 {
@@ -15,11 +15,14 @@ public:
 	CQueueStorage();
 	virtual ~CQueueStorage();
 
+	CQueueStorage(CQueueStorage const&) = delete;
+	CQueueStorage& operator=(CQueueStorage const&) = delete;
+
 	// Call before loading
 	bool BeginTransaction();
 
 	// Call after finishing loading
-	bool EndTransaction();
+	bool EndTransaction(bool rollback = false);
 
 	bool Clear(); // Also clears caches
 
@@ -30,12 +33,11 @@ public:
 	// > 0 = server id
 	//   0 = No server
 	// < 0 = failure.
-	wxLongLong_t GetServer(CServer& server, bool fromBeginning);
-	CServer GetNextServer();
+	int64_t GetServer(ServerWithCredentials& server, bool fromBeginning);
 
-	wxLongLong_t GetFile(CFileItem** pItem, wxLongLong_t server);
+	int64_t GetFile(CFileItem** pItem, int64_t server);
 
-	static wxString GetDatabaseFilename();
+	static std::wstring GetDatabaseFilename();
 
 private:
 	Impl* d_;
