@@ -5,6 +5,7 @@
 #include "search.h"
 #include "commandqueue.h"
 #include "filelistctrl.h"
+#include "file_utils.h"
 #include "ipcmutex.h"
 #include "Options.h"
 #include "queue.h"
@@ -14,6 +15,10 @@
 #include "window_state_manager.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/uri.hpp>
+
+#include <wx/clipbrd.h>
+
 class CRemoteSearchFileData final : public CDirentry
 {
 public:
@@ -47,10 +52,12 @@ inline int DoCmpName(CRemoteSearchFileData const& data1, CRemoteSearchFileData c
 	}
 
 	if (!res) {
-		if (data1.path < data2.path)
+		if (data1.path < data2.path) {
 			res = -1;
-		else if (data2.path < data1.path)
+		}
+		else if (data2.path < data1.path) {
 			res = 1;
+		}
 	}
 
 	return res;
@@ -75,16 +82,18 @@ inline int DoCmpName(CLocalSearchFileData const& data1, CLocalSearchFileData con
 	}
 
 	if (!res) {
-		if (data1.path < data2.path)
+		if (data1.path < data2.path) {
 			res = -1;
-		else if (data2.path < data1.path)
+		}
+		else if (data2.path < data1.path) {
 			res = 1;
+		}
 	}
 
 	return res;
 }
 
-class CSearchDialogFileList : public CFileListCtrl<CGenericFileData>
+class CSearchDialogFileList final : public CFileListCtrl<CGenericFileData>
 {
 	friend class CSearchDialog;
 	friend class CSearchSortType;
@@ -99,7 +108,7 @@ protected:
 
 	virtual int64_t ItemGetSize(int index) const;
 
-	CFileListCtrl<CGenericFileData>::CSortComparisonObject GetSortComparisonObject();
+	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
 
 	CSearchDialog *m_searchDialog;
 
@@ -179,133 +188,170 @@ int64_t CSearchDialogFileList::ItemGetSize(int index) const
 	}
 }
 
-CFileListCtrl<CGenericFileData>::CSortComparisonObject CSearchDialogFileList::GetSortComparisonObject()
+std::unique_ptr<CFileListCtrlSortBase> CSearchDialogFileList::GetSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
 	if (mode_ == CSearchDialog::search_mode::local) {
 		if (!m_sortDirection) {
-			if (m_sortColumn == 1)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 2)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 3)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 4)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-		
-			else
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
+			if (m_sortColumn == 1) {
+				return std::make_unique<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 2) {
+				return std::make_unique<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 3) {
+				return std::make_unique<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 4) {
+				return std::make_unique<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else {
+				return std::make_unique<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
 		}
 		else {
-			if (m_sortColumn == 1)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 2)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 3)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 4)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			
-			else
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
+			if (m_sortColumn == 1) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 2) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 3) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 4) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else {
+				return std::make_unique<CReverseSort<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
 		}
 	}
 	else {
 		if (!m_sortDirection) {
-			if (m_sortColumn == 1)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 2)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 3)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 4)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 5)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 6)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
+			if (m_sortColumn == 1) {
+				return std::make_unique<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 2) {
+				return std::make_unique<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 3) {
+				return std::make_unique<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 4) {
+				return std::make_unique<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 5) {
+				return std::make_unique<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 6) {
+				return std::make_unique<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else {
+				return std::make_unique<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
 		}
 		else {
-			if (m_sortColumn == 1)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 2)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 3)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 4)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 5)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else if (m_sortColumn == 6)
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			else
-				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this));
+			if (m_sortColumn == 1) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 2) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 3) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 4) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 5) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else if (m_sortColumn == 6) {
+				return std::make_unique<CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
+			else {
+				return std::make_unique<CReverseSort<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+			}
 		}
 	}
 }
 
 wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 {
-	if (item < 0 || item >= (int)m_indexMapping.size())
+	if (item < 0 || item >= (int)m_indexMapping.size()) {
 		return wxString();
+	}
 	int index = m_indexMapping[item];
 
 	if (mode_ == CSearchDialog::search_mode::local) {
 		CLocalSearchFileData const& entry = localFileData_[index];
-		if (!column)
+		if (!column) {
 			return entry.name;
-		else if (column == 1)
+		}
+		else if (column == 1) {
 			return entry.path.GetPath();
+		}
 		else if (column == 2) {
-			if (entry.is_dir() || entry.size < 0)
+			if (entry.is_dir() || entry.size < 0) {
 				return wxString();
-			else
+			}
+			else {
 				return CSizeFormat::Format(entry.size);
+			}
 		}
 		else if (column == 3) {
 			auto & data = m_fileData[index];
 			if (data.fileType.empty()) {
-				data.fileType = GetType(entry.name, entry.is_dir()).ToStdWstring();
+				data.fileType = GetType(entry.name, entry.is_dir());
 			}
 
 			return data.fileType;
 		}
-		else if (column == 4)
+		else if (column == 4) {
 			return CTimeFormat::Format(entry.time);
+		}
 	}
 	else {
 		CRemoteSearchFileData const& entry = remoteFileData_[index];
-		if (!column)
+		if (!column) {
 			return entry.name;
-		else if (column == 1)
+		}
+		else if (column == 1) {
 			return entry.path.GetPath();
+		}
 		else if (column == 2) {
-			if (entry.is_dir() || entry.size < 0)
+			if (entry.is_dir() || entry.size < 0) {
 				return wxString();
-			else
+			}
+			else {
 				return CSizeFormat::Format(entry.size);
+			}
 		}
 		else if (column == 3) {
 			auto & data = m_fileData[index];
 			if (data.fileType.empty()) {
-				if (entry.path.GetType() == VMS)
-					data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir()).ToStdWstring();
-				else
-					data.fileType = GetType(entry.name, entry.is_dir()).ToStdWstring();
+				if (entry.path.GetType() == VMS) {
+					data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
+				}
+				else {
+					data.fileType = GetType(entry.name, entry.is_dir());
+				}
 			}
 
 			return data.fileType;
 		}
-		else if (column == 4)
+		else if (column == 4) {
 			return CTimeFormat::Format(entry.time);
-		else if (column == 5)
+		}
+		else if (column == 5) {
 			return *entry.permissions;
-		else if (column == 6)
+		}
+		else if (column == 6) {
 			return *entry.ownerGroup;
+		}
 	}
 	return wxString();
 }
@@ -313,14 +359,16 @@ wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 int CSearchDialogFileList::OnGetItemImage(long item) const
 {
 	CSearchDialogFileList *pThis = const_cast<CSearchDialogFileList *>(this);
-	if (item < 0 || item >= (int)m_indexMapping.size())
+	if (item < 0 || item >= (int)m_indexMapping.size()) {
 		return -1;
+	}
 	int index = m_indexMapping[item];
 
 	int &icon = pThis->m_fileData[index].icon;
 
-	if (icon != -2)
+	if (icon != -2) {
 		return icon;
+	}
 
 	if (mode_ == CSearchDialog::search_mode::local) {
 		auto const& file = localFileData_[index];
@@ -361,6 +409,10 @@ EVT_MENU(XRCID("ID_MENU_SEARCH_DOWNLOAD"), CSearchDialog::OnDownload)
 EVT_MENU(XRCID("ID_MENU_SEARCH_UPLOAD"), CSearchDialog::OnUpload)
 EVT_MENU(XRCID("ID_MENU_SEARCH_EDIT"), CSearchDialog::OnEdit)
 EVT_MENU(XRCID("ID_MENU_SEARCH_DELETE"), CSearchDialog::OnDelete)
+EVT_MENU(XRCID("ID_MENU_SEARCH_GETURL"), CSearchDialog::OnGetUrl)
+EVT_MENU(XRCID("ID_MENU_SEARCH_GETURL_PASSWORD"), CSearchDialog::OnGetUrl)
+EVT_MENU(XRCID("ID_MENU_SEARCH_OPEN"), CSearchDialog::OnOpen)
+EVT_MENU(XRCID("ID_MENU_SEARCH_FILEMANAGER"), CSearchDialog::OnOpen)
 EVT_CHAR_HOOK(CSearchDialog::OnCharHook)
 EVT_RADIOBUTTON(XRCID("ID_LOCAL_SEARCH"), CSearchDialog::OnChangeSearchMode)
 EVT_RADIOBUTTON(XRCID("ID_REMOTE_SEARCH"), CSearchDialog::OnChangeSearchMode)
@@ -383,8 +435,9 @@ CSearchDialog::~CSearchDialog()
 
 bool CSearchDialog::Load()
 {
-	if (!wxDialogEx::Load(m_parent, _T("ID_SEARCH")))
+	if (!wxDialogEx::Load(m_parent, _T("ID_SEARCH"))) {
 		return false;
+	}
 
 	// XRCed complains if adding a status bar to a dialog, so do it here instead
 	CFilelistStatusBar* pStatusBar = new CFilelistStatusBar(this);
@@ -393,8 +446,9 @@ bool CSearchDialog::Load()
 
 	GetSizer()->Add(pStatusBar, 0, wxGROW);
 
-	if (!CreateListControl(filter_name | filter_size | filter_path | filter_date))
+	if (!CreateListControl(filter_name | filter_size | filter_path | filter_date)) {
 		return false;
+	}
 
 	m_results = new CSearchDialogFileList(this, 0);
 	ReplaceControl(XRCCTRL(*this, "ID_RESULTS", wxWindow), m_results);
@@ -453,12 +507,14 @@ void CSearchDialog::Run()
 			m_state.m_pCommandQueue->Cancel();
 			m_state.GetRemoteRecursiveOperation()->StopRecursiveOperation();
 		}
-		if (!m_original_dir.empty())
+		if (!m_original_dir.empty()) {
 			m_state.ChangeRemoteDir(m_original_dir);
+		}
 	}
 	else {
-		if (m_state.IsRemoteIdle() && !m_original_dir.empty())
+		if (m_state.IsRemoteIdle() && !m_original_dir.empty()) {
 			m_state.ChangeRemoteDir(m_original_dir);
+		}
 	}
 }
 
@@ -499,12 +555,14 @@ void CSearchDialog::OnStateChange(t_statechange_notifications notification, cons
 
 void CSearchDialog::ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> const& listing)
 {
-	if (!listing || listing->failed())
+	if (!listing || listing->failed()) {
 		return;
+	}
 
 	// Do not process same directory multiple times
-	if (!m_visited.insert(listing->path).second)
+	if (!m_visited.insert(listing->path).second) {
 		return;
+	}
 
 	int old_count = m_results->m_fileData.size();
 	int added_count = 0;
@@ -518,7 +576,7 @@ void CSearchDialog::ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> c
 		added_indexes.reserve(listing->GetCount());
 	}
 
-	CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = m_results->GetSortComparisonObject();
+	std::unique_ptr<CFileListCtrlSortBase> compare = m_results->GetSortComparisonObject();
 	for (unsigned int i = 0; i < listing->GetCount(); ++i) {
 		const CDirentry& entry = (*listing)[i];
 
@@ -535,7 +593,7 @@ void CSearchDialog::ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> c
 		data.icon = entry.is_dir() ? m_results->m_dirIcon : -2;
 		m_results->m_fileData.push_back(data);
 
-		auto insertPos = std::lower_bound(m_results->m_indexMapping.begin(), m_results->m_indexMapping.end(), old_count + added_count, compare);
+		auto insertPos = std::lower_bound(m_results->m_indexMapping.begin(), m_results->m_indexMapping.end(), old_count + added_count, SortPredicate(compare));
 		int const added_index = insertPos - m_results->m_indexMapping.begin();
 		m_results->m_indexMapping.insert(insertPos, old_count + added_count++);
 
@@ -548,12 +606,13 @@ void CSearchDialog::ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> c
 			added_indexes.insert(added_indexes_insert_pos, added_index);
 		}
 
-		if (entry.is_dir())
+		if (entry.is_dir()) {
 			m_results->GetFilelistStatusBar()->AddDirectory();
-		else
+		}
+		else {
 			m_results->GetFilelistStatusBar()->AddFile(entry.size);
+		}
 	}
-	compare.Destroy();
 
 	if (added_count) {
 		m_results->SetItemCount(old_count + added_count);
@@ -576,7 +635,7 @@ void CSearchDialog::ProcessDirectoryListing(CLocalRecursiveOperation::listing co
 		added_indexes.reserve(listing.files.size() + listing.dirs.size());
 	}
 
-	CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = m_results->GetSortComparisonObject();
+	std::unique_ptr<CFileListCtrlSortBase> compare = m_results->GetSortComparisonObject();
 
 	auto const& add_entry = [&](CLocalRecursiveOperation::listing::entry const& entry, bool dir) {
 		if (!CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, path, dir, entry.size, entry.attributes, entry.time)) {
@@ -593,7 +652,7 @@ void CSearchDialog::ProcessDirectoryListing(CLocalRecursiveOperation::listing co
 		data.icon = dir ? m_results->m_dirIcon : -2;
 		m_results->m_fileData.push_back(data);
 
-		auto insertPos = std::lower_bound(m_results->m_indexMapping.begin(), m_results->m_indexMapping.end(), old_count + added_count, compare);
+		auto insertPos = std::lower_bound(m_results->m_indexMapping.begin(), m_results->m_indexMapping.end(), old_count + added_count, SortPredicate(compare));
 		int const added_index = insertPos - m_results->m_indexMapping.begin();
 		m_results->m_indexMapping.insert(insertPos, old_count + added_count++);
 
@@ -620,7 +679,6 @@ void CSearchDialog::ProcessDirectoryListing(CLocalRecursiveOperation::listing co
 	for (auto const& dir : listing.dirs) {
 		add_entry(dir, true);
 	}
-	compare.Destroy();
 
 	if (added_count) {
 		m_results->SetItemCount(old_count + added_count);
@@ -661,12 +719,12 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 
 		CServerPath path;
 
-		const CServer* pServer = m_state.GetServer();
-		if (!pServer) {
+		ServerWithCredentials const& server = m_state.GetServer();
+		if (!server) {
 			wxMessageBoxEx(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
 			return;
 		}
-		path.SetType(pServer->GetType());
+		path.SetType(server.server.GetType());
 		if (!path.SetPath(xrc_call(*this, "ID_PATH", &wxTextCtrl::GetValue).ToStdWstring()) || path.empty()) {
 			wxMessageBoxEx(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
 			return;
@@ -698,7 +756,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 	m_visited.clear();
 
 	// Start
-	
+
 	if (localSearch) {
 		local_recursion_root root;
 		root.add_dir_to_visit(m_local_search_root);
@@ -769,18 +827,24 @@ void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 	if (!m_state.IsRemoteIdle() || !m_state.IsRemoteConnected()) {
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_UPLOAD"), false);
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_DOWNLOAD"), false);
-		pMenu->Enable(XRCID("ID_MENU_SEARCH_DELETE"), false);
+		if (m_results->mode_ == search_mode::remote) {
+			pMenu->Enable(XRCID("ID_MENU_SEARCH_DELETE"), false);
+		}
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_EDIT"), false);
 	}
 
 	bool const localSearch = m_results->mode_ == search_mode::local;
 	if (localSearch) {
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_DOWNLOAD"));
-		pMenu->Delete(XRCID("ID_MENU_SEARCH_DELETE"));
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_EDIT"));
+		pMenu->Delete(XRCID("ID_MENU_SEARCH_GETURL"));
+		pMenu->Delete(XRCID("ID_MENU_SEARCH_GETURL_PASSWORD"));
 	}
 	else {
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_UPLOAD"));
+		pMenu->Delete(XRCID("ID_MENU_SEARCH_OPEN"));
+		pMenu->Delete(XRCID("ID_MENU_SEARCH_FILEMANAGER"));
+		pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_MENU_SEARCH_GETURL" : "ID_MENU_SEARCH_GETURL_PASSWORD"));
 	}
 
 	PopupMenu(pMenu);
@@ -789,21 +853,24 @@ void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 
 
 
-class CSearchTransferDialog : public wxDialogEx
+class CSearchTransferDialog final : public wxDialogEx
 {
 public:
 	bool Run(wxWindow* parent, bool download, const wxString& path, int count_files, int count_dirs)
 	{
 		download_ = download;
 
-		if (!Load(parent, download ? _T("ID_SEARCH_DOWNLOAD") : _T("ID_SEARCH_UPLOAD")))
+		if (!Load(parent, download ? _T("ID_SEARCH_DOWNLOAD") : _T("ID_SEARCH_UPLOAD"))) {
 			return false;
+		}
 
 		wxString desc;
-		if (!count_dirs)
+		if (!count_dirs) {
 			desc.Printf(wxPLURAL("Selected %d file for transfer.", "Selected %d files for transfer.", count_files), count_files);
-		else if (!count_files)
+		}
+		else if (!count_files) {
 			desc.Printf(wxPLURAL("Selected %d directory with its contents for transfer.", "Selected %d directories with their contents for transfer.", count_dirs), count_dirs);
+		}
 		else {
 			wxString files = wxString::Format(wxPLURAL("%d file", "%d files", count_files), count_files);
 			wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", count_dirs), count_dirs);
@@ -818,8 +885,9 @@ public:
 			XRCCTRL(*this, "ID_REMOTEPATH", wxTextCtrl)->ChangeValue(path);
 		}
 
-		if (ShowModal() != wxID_OK)
+		if (ShowModal() != wxID_OK) {
 			return false;
+		}
 
 		return true;
 	}
@@ -895,20 +963,23 @@ void ProcessSelection(std::list<int> &selected_files, std::deque<Path> &selected
 
 	int sel = -1;
 	while ((sel = results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
-		if (sel > (int)results->indexMapping().size())
+		if (static_cast<size_t>(sel) >= results->indexMapping().size()) {
 			continue;
+		}
 		int index = results->indexMapping()[sel];
 
 		if (fileData[index].is_dir()) {
 			Path path = fileData[index].path;
 			path.ChangePath(fileData[index].name);
-			if (path.empty())
+			if (path.empty()) {
 				continue;
+			}
 
 			dirs.push_back(path);
 		}
-		else
+		else {
 			selected_files.push_back(index);
+		}
 	}
 
 	// Make sure that selected_dirs does not contain
@@ -932,8 +1003,9 @@ void ProcessSelection(std::list<int> &selected_files, std::deque<Path> &selected
 				break;
 			}
 		}
-		if (path_iter == selected_dirs.end())
+		if (path_iter == selected_dirs.end()) {
 			selected_files_new.push_back(sel_file);
+		}
 	}
 	selected_files.swap(selected_files_new);
 
@@ -971,8 +1043,8 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 		return;
 	}
 
-	CServer const* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -1004,7 +1076,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 
 		m_pQueue->QueueFile(!start, true,
 			entry.name, (localName != entry.name) ? localName : std::wstring(),
-			target_path, remote_path, *pServer, entry.size);
+			target_path, remote_path, server, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
 
@@ -1040,8 +1112,9 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 	}
 
 	CSearchTransferDialog dlg;
-	if (!dlg.Run(this, false, m_state.GetRemotePath().GetPath(), selected_files.size(), selected_dirs.size()))
+	if (!dlg.Run(this, false, m_state.GetRemotePath().GetPath(), selected_files.size(), selected_dirs.size())) {
 		return;
+	}
 
 	wxTextCtrl *pText = XRCCTRL(dlg, "ID_REMOTEPATH", wxTextCtrl);
 
@@ -1051,8 +1124,8 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 		return;
 	}
 
-	CServer const* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -1082,12 +1155,12 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 
 		m_pQueue->QueueFile(!start, false,
 			entry.name, (localName != entry.name) ? localName : std::wstring(),
-			local_path, target_path, *pServer, entry.size);
+			local_path, target_path, server, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
 
 	auto const mode = flatten ? CRecursiveOperation::recursive_transfer_flatten : CRecursiveOperation::recursive_transfer;
-	
+
 	for (auto const& dir : selected_dirs) {
 		CServerPath target_path = path;
 		if (!flatten && dir.HasParent()) {
@@ -1117,8 +1190,9 @@ void CSearchDialog::OnEdit(wxCommandEvent&)
 	std::list<int> selected_files;
 	ProcessSelection(selected_files, selected_dirs, m_results->remoteFileData_, m_results);
 
-	if (selected_files.empty() && selected_dirs.empty())
+	if (selected_files.empty() && selected_dirs.empty()) {
 		return;
+	}
 
 	if (!selected_dirs.empty()) {
 		wxMessageBoxEx(_("Editing directories is not supported"), _("Editing search results"), wxICON_EXCLAMATION);
@@ -1137,8 +1211,8 @@ void CSearchDialog::OnEdit(wxCommandEvent&)
 		return;
 	}
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -1148,69 +1222,93 @@ void CSearchDialog::OnEdit(wxCommandEvent&)
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
 
-		if (!dlg.Run())
+		if (!dlg.Run()) {
 			return;
+		}
 	}
 
 	for (auto const& item : selected_files) {
 		const CDirentry& entry = m_results->remoteFileData_[item];
 		const CServerPath path = m_results->remoteFileData_[item].path;
 
-		pEditHandler->Edit(CEditHandler::remote, entry.name, path, *pServer, entry.size, this);
+		pEditHandler->Edit(CEditHandler::remote, entry.name, path, server, entry.size, this);
 	}
 }
 
 void CSearchDialog::OnDelete(wxCommandEvent&)
 {
-	if (!m_state.IsRemoteIdle()) {
-		return;
-	}
-
-	if (m_results->mode_ != search_mode::remote) {
-		return;
-	}
-
-	// Find all selected files and directories
-	std::deque<CServerPath> selected_dirs;
-	std::list<int> selected_files;
-	ProcessSelection(selected_files, selected_dirs, m_results->remoteFileData_, m_results);
-
-	if (selected_files.empty() && selected_dirs.empty())
-		return;
-
-	wxString question;
-	if (selected_dirs.empty())
-		question.Printf(wxPLURAL("Really delete %d file from the server?", "Really delete %d files from the server?", selected_files.size()), selected_files.size());
-	else if (selected_files.empty())
-		question.Printf(wxPLURAL("Really delete %d directory with its contents from the server?", "Really delete %d directories with their contents from the server?", selected_dirs.size()), selected_dirs.size());
-	else {
-		wxString files = wxString::Format(wxPLURAL("%d file", "%d files", selected_files.size()), selected_files.size());
-		wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", selected_dirs.size()), selected_dirs.size());
-		question.Printf(_("Really delete %s and %s from the server?"), files, dirs);
-	}
-
-	if (wxMessageBoxEx(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES)
-		return;
-
-	for (auto const& file : selected_files) {
-		CDirentry const& entry = m_results->remoteFileData_[file];
-		std::deque<std::wstring> files_to_delete;
-		files_to_delete.push_back(entry.name);
-		m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_results->remoteFileData_[file].path, std::move(files_to_delete)));
-	}
-
-	for (auto path : selected_dirs) {
-		std::wstring segment;
-		if (path.HasParent()) {
-			segment = path.GetLastSegment();
-			path = path.GetParent();
+	if (m_results->mode_ == search_mode::remote) {
+		if (!m_state.IsRemoteIdle()) {
+			return;
 		}
-		recursion_root root(path, !path.HasParent());
-		root.add_dir_to_visit(path, segment);
-		m_state.GetRemoteRecursiveOperation()->AddRecursionRoot(std::move(root));
+
+		// Find all selected files and directories
+		std::deque<CServerPath> selected_dirs;
+		std::list<int> selected_files;
+		ProcessSelection(selected_files, selected_dirs, m_results->remoteFileData_, m_results);
+
+		if (selected_files.empty() && selected_dirs.empty()) {
+			return;
+		}
+
+		wxString question;
+		if (selected_dirs.empty()) {
+			question.Printf(wxPLURAL("Really delete %d file from the server?", "Really delete %d files from the server?", selected_files.size()), selected_files.size());
+		}
+		else if (selected_files.empty()) {
+			question.Printf(wxPLURAL("Really delete %d directory with its contents from the server?", "Really delete %d directories with their contents from the server?", selected_dirs.size()), selected_dirs.size());
+		}
+		else {
+			wxString files = wxString::Format(wxPLURAL("%d file", "%d files", selected_files.size()), selected_files.size());
+			wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", selected_dirs.size()), selected_dirs.size());
+			question.Printf(_("Really delete %s and %s from the server?"), files, dirs);
+		}
+
+		if (wxMessageBoxEx(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES) {
+			return;
+		}
+
+		for (auto const& file : selected_files) {
+			CRemoteSearchFileData const& entry = m_results->remoteFileData_[file];
+			std::deque<std::wstring> files_to_delete;
+			files_to_delete.push_back(entry.name);
+			m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(entry.path, std::move(files_to_delete)));
+		}
+
+		for (auto path : selected_dirs) {
+			std::wstring segment;
+			if (path.HasParent()) {
+				segment = path.GetLastSegment();
+				path = path.GetParent();
+			}
+			recursion_root root(path, !path.HasParent());
+			root.add_dir_to_visit(path, segment);
+			m_state.GetRemoteRecursiveOperation()->AddRecursionRoot(std::move(root));
+		}
+		ActiveFilters const filters; // Empty, recurse into everything
+		m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filters, m_original_dir);
+	}
+	else {
+		std::deque<CLocalPath> selected_dirs;
+		std::list<int> selected_files;
+		ProcessSelection(selected_files, selected_dirs, m_results->localFileData_, m_results);
+
+		if (selected_files.empty() && selected_dirs.empty()) {
+			return;
+		}
+
+		std::list<fz::native_string> pathsToDelete;
+		for (auto const& file : selected_files) {
+			CLocalSearchFileData const& entry = m_results->localFileData_[file];
+			pathsToDelete.push_back(fz::to_native(entry.path.GetPath() + entry.name));
+		}
+		for (auto path : selected_dirs) {
+			pathsToDelete.push_back(fz::to_native(path.GetPath()));
+		}
+
+		gui_recursive_remove rmd(this);
+		rmd.remove(pathsToDelete);
 	}
-	ActiveFilters const filters; // Empty, recurse into everything
-	m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filters, m_original_dir);
 }
 
 void CSearchDialog::OnCharHook(wxKeyEvent& event)
@@ -1257,8 +1355,9 @@ void CSearchDialog::LoadConditions()
 	if (!filter)
 		return;
 
-	if (!CFilterManager::LoadFilter(filter, m_search_filter))
+	if (!CFilterManager::LoadFilter(filter, m_search_filter)) {
 		m_search_filter = CFilter();
+	}
 }
 
 void CSearchDialog::SaveConditions()
@@ -1273,8 +1372,9 @@ void CSearchDialog::SaveConditions()
 	}
 
 	pugi::xml_node filter;
-	while ((filter = document.child("Filter")))
+	while ((filter = document.child("Filter"))) {
 		document.remove_child(filter);
+	}
 	filter = document.append_child("Filter");
 
 	CFilterManager::SaveFilter(filter, m_search_filter);
@@ -1302,3 +1402,137 @@ void CSearchDialog::OnChangeSearchMode(wxCommandEvent&)
 	}
 
 }
+
+void CSearchDialog::OnGetUrl(wxCommandEvent& event)
+{
+	if (m_results->mode_ != search_mode::remote) {
+		return;
+	}
+
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
+		wxBell();
+		return;
+	}
+
+	if (!wxTheClipboard->Open()) {
+		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	wxString const url = server.Format((event.GetId() == XRCID("ID_MENU_SEARCH_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+
+	auto getUrl = [](wxString const& serverPart, CServerPath const& path, std::wstring const& name) {
+		wxString url = serverPart;
+
+		auto const pathPart = fz::percent_encode_w(path.FormatFilename(name, false), true);
+		if (!pathPart.empty() && pathPart[0] != '/') {
+			url += '/';
+		}
+		url += pathPart;
+
+		return url;
+	};
+
+	wxString urls;
+
+	int sel = -1;
+	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) >= 0) {
+		if (static_cast<size_t>(sel) >= m_results->m_indexMapping.size()) {
+			continue;
+		}
+		int index = m_results->m_indexMapping[sel];
+
+		auto const& entry = m_results->remoteFileData_[index];
+		urls += getUrl(url, entry.path, entry.name);
+#ifdef __WXMSW__
+		urls += _T("\r\n");
+#else
+		urls += _T("\n");
+#endif
+	}
+
+	wxTheClipboard->SetData(new wxURLDataObject(urls));
+
+	wxTheClipboard->Flush();
+	wxTheClipboard->Close();
+}
+
+void CSearchDialog::OnOpen(wxCommandEvent & event)
+{
+	std::list<CLocalSearchFileData> selected_item_list;
+
+	int sel = -1;
+	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
+		if (static_cast<size_t>(sel) >= m_results->indexMapping().size()) {
+			continue;
+		}
+		int index = m_results->indexMapping()[sel];
+
+		selected_item_list.push_back(m_results->localFileData_[index]);
+	}
+
+	CEditHandler* pEditHandler = CEditHandler::Get();
+	if (!pEditHandler) {
+		wxBell();
+		return;
+	}
+
+	if (selected_item_list.empty()) {
+		wxBell();
+		return;
+	}
+
+	if (selected_item_list.size() > 10) {
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files or directories to open, do you really want to continue?"));
+
+		if (!dlg.Run()) {
+			return;
+		}
+	}
+
+	bool const open = event.GetId() == XRCID("ID_MENU_SEARCH_OPEN");
+
+	for (auto const& data : selected_item_list) {
+
+		std::wstring const fn = data.path.GetPath() + data.name;
+		if (data.dir) {
+			OpenInFileManager(fn);
+			continue;
+		}
+
+		if (!open) {
+			OpenInFileManager(data.path.GetPath());
+			continue;
+		}
+
+		if (wxLaunchDefaultApplication(fn, 0)) {
+			continue;
+		}
+		bool program_exists = false;
+		wxString cmd = GetSystemOpenCommand(fn, program_exists);
+		if (cmd.empty()) {
+			auto pos = data.name.find('.');
+			if (pos == std::wstring::npos || (pos == 0 && data.name.find('.', 1) == std::wstring::npos)) {
+				cmd = pEditHandler->GetOpenCommand(fn, program_exists);
+			}
+		}
+		if (cmd.empty()) {
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn), _("Opening failed"), wxICON_EXCLAMATION);
+			continue;
+		}
+		if (!program_exists) {
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn, cmd);
+			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+			continue;
+		}
+
+		if (wxExecute(cmd)) {
+			continue;
+		}
+
+		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn), _("Opening failed"), wxICON_EXCLAMATION);
+	}
+}
