@@ -1,7 +1,7 @@
-#ifndef __OPTIONSPAGE_CONNECTION_ACTIVE_H__
-#define __OPTIONSPAGE_CONNECTION_ACTIVE_H__
+#ifndef FILEZILLA_INTERFACE_OPTIONSPAGE_CONNECTION_ACTIVE_HEADER
+#define FILEZILLA_INTERFACE_OPTIONSPAGE_CONNECTION_ACTIVE_HEADER
 
-class COptionsPageConnectionActive : public COptionsPage
+class COptionsPageConnectionActive final : public COptionsPage
 {
 public:
 	virtual wxString GetResourceName() { return _T("ID_SETTINGS_CONNECTION_ACTIVE"); }
@@ -12,8 +12,8 @@ public:
 protected:
 	virtual void SetCtrlState();
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnRadioOrCheckEvent(wxCommandEvent& event);
 };
 
-#endif //__OPTIONSPAGE_CONNECTION_ACTIVE_H__
+#endif
