@@ -1,5 +1,5 @@
-#ifndef __SITEMANAGER_DIALOG_H__
-#define __SITEMANAGER_DIALOG_H__
+#ifndef FILEZILLA_INTERFACE_SITEMANAGER_DIALOG_HEADER
+#define FILEZILLA_INTERFACE_SITEMANAGER_DIALOG_HEADER
 
 #include "dialogex.h"
 #include "sitemanager.h"
@@ -16,7 +16,7 @@ class CSiteManagerDialog final : public wxDialogEx
 public:
 	struct _connected_site
 	{
-		CServer server;
+		ServerWithCredentials server;
 		std::wstring old_path;
 		std::wstring new_path;
 	};
@@ -26,22 +26,23 @@ public:
 	virtual ~CSiteManagerDialog();
 
 	// Creation. If pServer is set, it will cause a new item to be created.
-	bool Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, const CServer* pServer = 0);
+	bool Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, ServerWithCredentials const* pServer = 0);
+
+	bool GetServer(Site& data, Bookmark& bookmark);
 
-	bool GetServer(Site& data);
-	
 protected:
 	// Creates the controls and sizers
 	void CreateControls(wxWindow* parent);
 
 	bool Verify();
 	bool UpdateItem();
-	bool UpdateServer(Site &server, const wxString& name);
-	bool UpdateBookmark(Bookmark &bookmark, const CServer& server);
+	bool UpdateServer(Site &server, wxString const& name);
+	bool UpdateBookmark(Bookmark &bookmark, ServerWithCredentials const& server);
 	bool Load();
 	bool Save(pugi::xml_node element = pugi::xml_node(), wxTreeItemId treeId = wxTreeItemId());
 	bool SaveChild(pugi::xml_node element, wxTreeItemId child);
 	void SetCtrlState();
+	void SetLogonTypeCtrlState();
 	bool LoadDefaultSites();
 
 	void SetProtocol(ServerProtocol protocol);
@@ -52,8 +53,8 @@ protected:
 
 	wxString FindFirstFreeName(const wxTreeItemId &parent, const wxString& name);
 
-	void AddNewSite(wxTreeItemId parent, const CServer& server, bool connected = false);
-	void CopyAddServer(const CServer& server);
+	void AddNewSite(wxTreeItemId parent, ServerWithCredentials const& server, bool connected = false);
+	void CopyAddServer(ServerWithCredentials const& server);
 
 	void AddNewBookmark(wxTreeItemId parent);
 
@@ -89,6 +90,7 @@ protected:
 	void OnNewBookmark(wxCommandEvent&);
 	void OnBookmarkBrowse(wxCommandEvent&);
 	void OnKeyFileBrowse(wxCommandEvent&);
+	void OnGenerateEncryptionKey(wxCommandEvent&);
 
 	void SetControlVisibility(ServerProtocol protocol, LogonType type);
 
@@ -109,9 +111,16 @@ protected:
 	wxNotebook *m_pNotebook_Site{};
 	wxNotebook *m_pNotebook_Bookmark{};
 
+	wxNotebookPage *m_pCharsetPage{};
+	wxString m_charsetPageText;
+	size_t m_charsetPageIndex = -1;
+	size_t m_totalPages = -1;
+
 	std::vector<_connected_site> *m_connected_sites{};
 
 	bool m_is_deleting{};
+
+	std::map<ServerProtocol, int> mainProtocolListIndex_;
 };
 
-#endif //__SITEMANAGER_DIALOG_H__
+#endif
