@@ -45,7 +45,7 @@ bool CTheme::Load(std::wstring const& theme)
 	if (!theme.empty()) {
 		path_ += L"/";
 	}
-	
+
 	CXmlFile xml(path_ + L"theme.xml");
 	auto xtheme = xml.Load().child("Theme");
 	if (!xtheme) {
@@ -215,7 +215,7 @@ std::vector<wxBitmap> CTheme::GetAllImages(wxSize const& size)
 	wxLogNull null;
 
 	std::vector<wxBitmap> ret;
-	
+
 	for (auto const& psize : sizes_) {
 		if (psize.second) {
 			fz::local_filesys fs;
@@ -402,7 +402,7 @@ std::vector<wxBitmap> CThemeProvider::GetAllImages(std::wstring const& theme, wx
 
 		it = themes_.insert(std::make_pair(theme, t)).first;
 	}
-	
+
 	return it->second.GetAllImages(size);
 }
 
@@ -468,7 +468,13 @@ void CThemeProvider::OnOptionsChanged(changed_options_t const&)
 wxSize CThemeProvider::GetIconSize(iconSize size, bool userScaled)
 {
 	int s;
-	if (size == iconSizeSmall) {
+	if (size == iconSizeTiny) {
+		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X) * 3 / 4;
+		if (s <= 0) {
+			s = 12;
+		}
+	}
+	else if (size == iconSizeSmall) {
 		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X);
 		if (s <= 0) {
 			s = 16;
@@ -518,3 +524,14 @@ wxSize CThemeProvider::GetIconSize(iconSize size, bool userScaled)
 
 	return ret;
 }
+
+double CThemeProvider::GetUIScaleFactor()
+{
+	int x = GetIconSize(iconSizeSmall).x;
+	if (!x) {
+		return 1.;
+	}
+	else {
+		return x / 16.;
+	}
+}
