@@ -7,11 +7,13 @@
 
 #include <wx/timer.h>
 
+#include <functional>
+
 struct build
 {
-	wxString url_;
-	wxString version_;
-	wxString hash_;
+	std::wstring url_;
+	std::wstring version_;
+	std::wstring hash_;
 	int64_t size_{-1};
 };
 
@@ -47,21 +49,21 @@ enum class UpdaterState
 class CUpdateHandler
 {
 public:
-	virtual void UpdaterStateChanged( UpdaterState s, build const& v ) = 0;
+	virtual void UpdaterStateChanged(UpdaterState s, build const& v) = 0;
 };
 
 class CFileZillaEngineContext;
-class CUpdater : public wxEvtHandler, private EngineNotificationHandler
+class CUpdater final : public wxEvtHandler, private EngineNotificationHandler
 {
 public:
-	CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context);
+	CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context, std::function<void(CActiveNotification const&)> const& activityNotificationHandler);
 	virtual ~CUpdater();
 
 	// 2-Stage initialization
 	void Init();
 
-	void AddHandler( CUpdateHandler& handler );
-	void RemoveHandler( CUpdateHandler& handler );
+	void AddHandler(CUpdateHandler& handler);
+	void RemoveHandler(CUpdateHandler& handler);
 
 	UpdaterState GetState() const { return state_; }
 	build AvailableBuild() const { return version_information_.available_; }
@@ -83,16 +85,16 @@ public:
 	void RunIfNeeded();
 
 protected:
-	int Download(wxString const& url, std::wstring const& local_file = std::wstring());
+	int Download(std::wstring const& url, std::wstring const& local_file = std::wstring());
 	int ContinueDownload();
 
 	void AutoRunIfNeeded();
 	bool Run();
 
-	bool CreateConnectCommand(wxString const& url);
-	bool CreateTransferCommand(wxString const& url, std::wstring const& local_file);
+	bool CreateConnectCommand(std::wstring const& url);
+	bool CreateTransferCommand(std::wstring const& url, std::wstring const& local_file);
 
-	wxString GetUrl();
+	std::wstring GetUrl();
 	void ProcessNotification(std::unique_ptr<CNotification> && notification);
 	void ProcessOperation(COperationNotification const& operation);
 	void ProcessData(CDataNotification& dataNotification);
@@ -100,13 +102,13 @@ protected:
 	UpdaterState ProcessFinishedDownload();
 	UpdaterState ProcessFinishedData(bool can_download);
 
-	bool VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum);
+	bool VerifyChecksum(wxString const& file, int64_t size, std::wstring const& checksum);
 
 	std::wstring GetTempFile() const;
 	wxString GetFilename(wxString const& url) const;
 	wxString GetLocalFile(build const& b, bool allow_existing);
 
-	void SetState( UpdaterState s );
+	void SetState(UpdaterState s);
 
 	virtual void OnEngineEvent(CFileZillaEngine* engine);
 	void DoOnEngineEvent(CFileZillaEngine* engine);
@@ -130,6 +132,8 @@ protected:
 	wxTimer update_timer_;
 
 	std::deque<std::unique_ptr<CCommand>> pending_commands_;
+
+	std::function<void(CActiveNotification const&)> activityNotificationHandler_;
 };
 
 #endif //FZ_MANUALUPDATECHECK
